;##############################################################################################
;#            Nome: SpiDEr10 (Spider's Disasm Engine release 1.0)                             #
;#        Versione: 1.0                                                                       #
;#          Autore: Spider                                                                    #
;# Ultima modifica: 11/06/2002                                                                #
;##############################################################################################
;# SpiDEr 1.0  una disasm engine che supporta le istruzioni dei processori Intel Pentium e   #
;# compatibili incluse le istruzioni FPU ed MMX, ma non le istruzioni SSE ed SSE2. Questa DLL #
;# esporta una sola funzione, Disasm, che si occupa di disassemblare una singola istruzione.  #
;# Disasm ritorna in eax la lunghezza in bytes dell'istruzione disassemblata.                 #
;##############################################################################################

.586p
.model flat,stdcall
option casemap:none

include \masm32\include\windows.inc

include SpiDErData.asm

Disasm           proto :DWORD,:DWORD,:DWORD,:DWORD,:DWORD

.code

DllEntry proc hInstDLL:HINSTANCE, reason:DWORD, reservedvalue:DWORD 
        mov eax,TRUE
        ret 
DllEntry Endp

;Disassembla una istruzione.
;Richiede 5 parametri:
;source: offset dell'istruzione da disassemblare
;dest: offset del buffer in cui sar salvata l'istruzione disassemblata
;realaddress: offset REALE dell'istruzione da disassemblare (utilizzato solo per jmps, calls, ecc.)
;OpSizeAttr: default operand size attribute (DE_OPSIZERESET = 16 bit / DE_OPSIZESET = 32 bit)
;AddrSizeAttr: default address size attribute (DE_ADDRSIZERESET = 16 bit / DE_ADDRSIZESET = 32 bit)
;
;Restituisce in eax la lunghezza dell'istruzione

align dword

Disasm proc uses ebx esi edi source:DWORD, dest:DWORD, realaddress:DWORD, OpSizeAttr:DWORD, AddrSizeAttr:DWORD

    LOCAL len:DWORD         ;larghezza in bytes dell'istruzione

    LOCAL segoverride:DWORD ;memorizza l'eventuale segment-override prefix


    mov [len],0
    mov [segoverride],0


    cld                     ;azzera il direction flag

    mov ebx,[source]        ;ebx = offset dell'istruzione da disassemblare

    mov eax,offset onebytenames
    push offset onebytesyntax

loc1:
    movzx edx,byte ptr[ebx] ;dl = primo byte dell'opcode (per le single-byte) o secondo (per le istruzioni two-byte)


loc2:
    mov edi,dword ptr[edx*4 + eax] ;edi = offset del nome dell'istruzione
    test edi,edi ;se edi  zero...
    jz no_name   ;...non copia il nome in [dest]
    
    ;--> Copia il nome dell'istruzione in [dest]
    push edi

    or ecx,-1
    xor eax,eax
    repnz scasb
    not ecx
    dec ecx           ;ecx = larghezza del nome dell'istruzione (escluso il terminatore)

    pop esi           ;esi = offset del nome dell'istruzione
    mov edi,[dest]    ;edi = offset del buffer di destinazione
    rep movsb         ;copia il nome dell'istruzione nel buffer di destinazione
    ;<--

    inc [len]

no_name:
    pop ecx               ;ecx = offset della lookup table della sintassi delle istruzioni

    mov eax,[edx*8 + ecx] ;eax = Opcode Flags (prima dword) nella lookup table

    test eax,eax
    jz NoOperands

    cmp eax,-1
    jz Invalid
    
    test eax,DE_X
    jnz Escape

    test eax,DE_A
    jnz DirectAddress

    test eax,DE_J
    jnz RelativeAddress
    
    test eax,DE_O
    jnz DirectOffset

    test eax,DE_R
    jnz extension

    test eax,DE_PREFIX
    jnz prefix

    test eax,DE_F
    jnz changing
    test eax,DE_F2
    jnz changing2
    

General: ;istruzione con sintassi generica. Pu avere il ModR/M byte, ha 1 o pi operandi
         ;la cui dimensione pu essere varia, ecc. E' il formato della maggioranza delle
         ;istruzioni.

        test eax,DE_M  ;Se  presente il ModR/M byte...
        jz @F          
        inc [len]      ;...incrementa [len]
    @@:

        lea esi,[edx*8 + ecx] ;esi = offset dell'entry dell'istruzione nella lookup table
        inc ebx
        mov dl,[esi + 4]      ;dl = quinto byte dell'entry

        xor ecx,ecx
    operandcicle:

        push ecx

        mov dh, byte ptr[esi + 5 + ecx] ;dh = flags dell'operando

        cmp dh,DE_OP_one
        jz op_1

        cmp dh,DE_OP_CL
        jz op_CL

        test dh,DE_OP_reg
        jnz op_reg

        test dh,DE_OP_modrm
        jnz op_modrm

        test dh,DE_OP_a
        jnz op_accumulator

        test dh,DE_OP_o
        jnz op_opcode

        op_immediate: ;l'operando  un valore immediato
            test eax,DE_M ;Se  presente il ModR/M
            jz @F
            inc ebx       ;...incrementa ebx
          @@:
            push ecx
            and dh,0111b
            cmp dh,DE_OP_b
            jz imm_byte
;            cmp dh,DE_OP_c
;            jz imm_byte_or_word
            cmp dh,DE_OP_w
            jz imm_word
            cmp dh,DE_OP_d
            jz imm_dword
            cmp dh,DE_OP_v
            jz imm_word_or_dword
          imm_byte: ;l'immediato  di 1 byte
            mov al,byte ptr[ebx]
            call ConvertByteToHex
            inc [len]
            inc ebx
            jmp imm_end
;          imm_byte_or_word: ;l'immediato  di 1 byte o 1 word, a seconda 
;                            ;dell' operand-size attribute
;            mov ecx,[OpSizeAttr]
;            jecxz imm_byte
          imm_word: ;l'immediato  di 1 word
            mov ax,word ptr[ebx]
            call ConvertWordToHex
            add [len],2
            add ebx,2
            jmp imm_end
          imm_word_or_dword: ;l'immediato  di 1 word o 1 dword, a seconda
                             ;dell'operand-size attribute
            mov ecx,[OpSizeAttr]
            jecxz imm_word
          imm_dword: ;l'immediato  di 1 dword
            mov eax,dword ptr[ebx]
            call ConvertDwordToHex
            add [len],4
            add ebx,4
            
          imm_end:
            pop ecx
            jmp @F

        op_1: ;l'operando  "1"
            mov byte ptr[edi],'1'
            inc edi
            jmp @F
        op_CL: ;l'operando  "CL"
            mov word ptr[edi],"LC"
            add edi,2
            jmp @F
        
        op_reg: ;l'operando  specificato dal reg field del ModR/M byte
            push edi
            push [OpSizeAttr]
            movzx edi,dh
            push edi
            push eax
            push ebx
            call ManageRegField
            jmp @F

        op_modrm: ;l'operando  specificato da Mod e R/M del ModR/M byte e dal SIB byte
            push eax

            push edi
            push [segoverride]  ;segcode
            push [AddrSizeAttr] ;addrsize
            push [OpSizeAttr]   ;opsize
            movzx edi, dh
            push edi            ;OpFlags
            push eax            ;Flags
            push ebx            ;Address
            call ManageModRM

            add [len],eax ;Importante! Dobbiamo tener conto dell'eventuale SIB
                          ;e dell'eventuale displacement!

            pop eax
            jmp @F
            
        op_accumulator: ;l'operando  al, ax o eax
            and dh,0111b
            cmp dh,DE_OP_b
            jz acc_byte
;            cmp dh,DE_OP_c
;            jz acc_byte_or_word
            cmp dh,DE_OP_w
            jz acc_word
            cmp dh,DE_OP_d
            jz acc_dword
            cmp dh,DE_OP_v
            jz acc_word_or_dword
          acc_byte: ;L'operando  AL
            mov word ptr[edi],"LA"
            add edi,2
            jmp @F
;          acc_byte_or_word:
;            mov ecx,[OpSizeAttr] ;se l'operand-size attribute  0 (16 bit)
;            jecxz acc_byte       ;l'operando  1 byte (AL)
;                                 ;altrimenti  1 word (AX)
          acc_word: ;L'operando  AX
            mov word ptr[edi],"XA"
            add edi,2            
            jmp @F
          acc_word_or_dword:     ;se l'operand-size attribute  0 (16 bit)
            cmp[OpSizeAttr],0    ;l'operando  1 word (AX)            
            jz acc_word          ;altrimenti  1 dword (EAX)
            
          acc_dword: ;L'operando  EAX
            mov dword ptr[edi],"XAE"
            add edi,3            
            jmp @F

        op_opcode: ;L'operando  un general register definito dai
                   ;tre bit meno significativi dell'opcode
            push ecx
            push esi
            
            and dh,0111b
            cmp dh,DE_OP_b
            jz op_byte
;            cmp dh,DE_OP_c
;            jz op_byte_or_word
            cmp dh,DE_OP_w
            jz op_word
            cmp dh,DE_OP_d
            jz op_dword
            cmp dh,DE_OP_v
            jz op_word_or_dword
          op_byte:
            mov esi,offset DE_gen_regs_8
            mov ecx,2
            jmp op_copy_reg
;          op_byte_or_word:
;            mov ecx,[OpSizeAttr]
;            jecxz op_byte
          op_word:
            mov esi,offset DE_gen_regs_16
            mov ecx,2
            jmp op_copy_reg
          op_word_or_dword:
            mov ecx,[OpSizeAttr]
            jecxz op_word
          op_dword:      
            mov esi,offset DE_gen_regs_32
            mov ecx,3
            
          op_copy_reg:
            movzx eax,byte ptr[ebx-1]    ;[ebx - 1] contiene l'opcode
            and al,00000111b           
            lea esi,[esi + eax*8]
            rep movsb
            pop esi
            pop ecx

            dec ebx ;senza di questo, si avrebbero problemi con la gestione dell'immediato
                    ;per queste due istruzioni:
                    ; B0+r8  - MOV r8,imm8
                    ; B8+r32 - MOV r32,imm32
            
        
     @@:mov byte ptr[edi],',' ;aggiunge la virgola di separazione degli operandi

        pop ecx

        inc edi
        inc ecx
        cmp cl,dl
        jb operandcicle
        mov byte ptr[edi-1],0 ;aggiunge il terminatore
        mov eax,[len]
        ret

NoOperands: ;istruzione senza operandi
    mov eax,[len]
    mov byte ptr[edi],0 ;adds the termination char to the string
    ret

changing: ;l'istruzione segue 2 tabelle diverse, a seconda che l'operand-size
          ;attribute sia settato o meno (esempio: scasw/scasd)
     mov eax,dword ptr[OpSizeAttr]
     test eax,eax
     jz c_16
   c_32:
     mov eax,1
     jmp @F
   c_16:
     xor eax,eax
   @@:
    mov esi,[edx*8+ecx+4]
    mov esi,[esi+4]
    lea esi,[esi + eax*8]
    push esi

    mov esi,[edx*8+ecx+4]
    mov esi,[esi]
    lea esi,[esi + eax*4]

    mov eax,esi


    xor edx,edx
    jmp loc2

changing2: ;l'istruzione segue 2 tabelle diverse, a seconda che l'address-size
           ;attribute sia settato o meno (l'unica istruzione cos  jcxz/jecxz)
     mov eax,dword ptr[AddrSizeAttr]
     test eax,eax
     jz c2_16
   c2_32:
     mov eax,1
     jmp @F
   c2_16:
     xor eax,eax
   @@:
    mov esi,[edx*8+ecx+4]
    mov esi,[esi+4]
    lea esi,[esi + eax*8]
    push esi

    mov esi,[edx*8+ecx+4]
    mov esi,[esi]
    lea esi,[esi + eax*4]

    mov eax,esi


    xor edx,edx
    jmp loc2


DirectAddress: ;l'istruzione contiene un puntatore far di seguito all'opcode.
               ;(es. call (9A) e JMP (EA)
    inc ebx
    mov ecx,[OpSizeAttr]
    jecxz da_far_16_bit
  da_far_32_bit:
    movzx eax,word ptr[ebx+4]
    call ConvertWordToHex
    mov byte ptr[edi],':'
    inc edi
    mov eax,dword ptr[ebx]
    call ConvertDwordToHex

    mov byte ptr [edi],0

    mov eax,[len]
    add eax,6
    ret
  da_far_16_bit:
    movzx eax,word ptr[ebx+2]
    call ConvertWordToHex
    mov byte ptr[edi],':'
    inc edi
    mov ax,word ptr[ebx]
    call ConvertWordToHex

    mov byte ptr [edi],0

    mov eax,[len]
    add eax,4
    ret


RelativeAddress: ;istruzione con indirizzo relativo (call, jmp, jcc, loop, ecc.)

    mov eax,[edx*8 + ecx + 4] ;eax = Opcode Flags (2nd dword) in lookup table

    inc ebx

    cmp eax,DE_OP_v
    jz rel_word_or_dword
;    cmp eax,DE_OP_b
;    jz rel_byte

    rel_byte:
        movsx eax,byte ptr[ebx]
        add eax,[realaddress]
        add eax,[len]
        inc eax
        call ConvertDwordToHex

        mov byte ptr [edi],0

        mov eax,[len]
        inc eax
        ret
                              ;else it's a word operand
    rel_word:
        movsx eax,word ptr[ebx]
        add eax,[realaddress]
        add eax,[len]
        add eax,2
        call ConvertDwordToHex

        mov byte ptr [edi],0

        mov eax,[len]
        add eax,2
        ret
    rel_word_or_dword:
        mov ecx,[OpSizeAttr]
        jecxz rel_word
    rel_dword:      
        mov eax,dword ptr[ebx]
        add eax,[realaddress]
        add eax,[len]
        add eax,4
        call ConvertDwordToHex

        mov byte ptr [edi],0

        mov eax,[len]
        add eax,4
        ret

DirectOffset:
   ;L'operando  un operando di memoria codificato nei bytes che seguono l'opcode.
   ;Solo 4 istruzioni hanno questa sintassi:
   ; A0 - MOV AL, [ADDRESS]
   ; A1 - MOV eAX,[ADDRESS]
   ; A2 - MOV [ADDRESS],AL
   ; A3 - MOV [ADDRESS],eAX

    cmp dl,0A1h
    jb DO_A0
    jz DO_A1
    cmp dl,0A2h
    jz DO_A2
    jmp DO_A3
  DO_A0:

    mov dword ptr[edi],",LA"

    add edi,3

    ;--> Gestione dell'eventuale segment-override prefix
    mov esi,[segoverride]
    test esi,esi
    jz @F
    lea esi,[esi*4 + offset DE_segoverride_table]
    movsd
    dec edi
    @@:
    ;<--
    
    mov al,'['
    stosb
    mov ecx,[AddrSizeAttr]
    jecxz DO_A0_16bit
   DO_A0_32bit:
    mov eax,dword ptr[ebx+1]
    call ConvertDwordToHex
    add [len],4
    jmp @F
   DO_A0_16bit:
    mov ax,word ptr[ebx+1]
    call ConvertWordToHex
    add [len],2
   @@:
    mov al,"]"
    stosb

    xor eax,eax
    stosb         ;adds the termination string character

    mov eax,[len]
    ret
  DO_A1:
    mov ecx,[OpSizeAttr]
    jecxz DO_A1_OP16
   DO_A1_OP32:
    mov dword ptr[edi],",XAE"
    add edi,4
    jmp @F
   DO_A1_OP16:
    mov dword ptr[edi],",XA"
    add edi,3
   @@:


    ;--> Gestione dell'eventuale segment-override prefix
    mov esi,[segoverride]
    test esi,esi
    jz @F
    lea esi,[esi*4 + offset DE_segoverride_table]
    movsd
    dec edi
    @@:
    ;<--

    mov al,'['
    stosb
    
    mov ecx,[AddrSizeAttr]
    jecxz DO_A1_ADDR_16
   DO_A1_ADDR_32:
    mov eax,dword ptr[ebx+1]
    call ConvertDwordToHex
    add [len],4
    jmp @F
   DO_A1_ADDR_16:
    mov ax,word ptr[ebx+1]
    call ConvertWordToHex
    add [len],2
   @@:
    mov al,"]"
    stosb

    mov byte ptr[edi],0

    mov eax,[len]
    ret
  DO_A2:

    ;--> Gestione dell'eventuale segment-override prefix
    mov esi,[segoverride]
    test esi,esi
    jz @F
    lea esi,[esi*4 + offset DE_segoverride_table]
    movsd
    dec edi
    @@:
    ;<--

    mov al,'['
    stosb
    mov ecx,[AddrSizeAttr]
    jecxz DO_A2_16bit
   DO_A2_32bit:
    mov eax,dword ptr[ebx+1]
    call ConvertDwordToHex
    add [len],4
    jmp @F
   DO_A2_16bit:
    mov ax,word ptr[ebx+1]
    call ConvertWordToHex
    add [len],2
   @@:
    mov ax,",]"
    stosw


    mov dword ptr[edi],"LA"

    mov eax,[len]
    ret
  DO_A3:

    ;--> Gestione dell'eventuale segment-override prefix
    mov esi,[segoverride]
    test esi,esi
    jz @F
    lea esi,[esi*4 + offset DE_segoverride_table]
    movsd
    dec edi
    @@:
    ;<--

    mov al,'['
    stosb

    mov ecx,[AddrSizeAttr]
    jecxz DO_A3_ADDR_16
   DO_A3_ADDR_32:
    mov eax,dword ptr[ebx+1]
    call ConvertDwordToHex
    add [len],4
    jmp @F
   DO_A3_ADDR_16:
    mov ax,word ptr[ebx+1]
    call ConvertWordToHex
    add [len],2
   @@:
    mov ax,',]'
    stosw

    mov ecx,[OpSizeAttr]
    jecxz DO_A3_OP16
   DO_A3_OP32:
    mov dword ptr[edi],"XAE"
    jmp @F
   DO_A3_OP16:
    mov dword ptr[edi],"XA"
   @@:

    mov eax,[len]
    ret

extension: ;L'opcode contiene un estensione nel reg field del ModR/M byte
    mov eax,[edx*8+ecx+4]
    push [eax]
    mov eax,[eax+4]
    mov dl,byte ptr[ebx+1] ;dl = ModR/M
    shr dl,3
    and dl,111b            ;dl = reg field
    jmp loc2

prefix: ;L'istruzione contiene un prefisso (es: F3 (REP) o 66 (address-size override))
    inc [len]

    inc ebx
    inc [source]

    cmp dl,0F0h
    jae group1  ;prefisso del primo gruppo
    cmp dl,066h
    jb group2   ;prefisso di segment-override
    je group3   ;prefisso di operand-size override
    jmp group4  ;prefisso di address-size override
    
  group1: ;Lock or Repeat Prefix
    and dl,0Fh

    lea edi,[edx*8 + offset DE_prefixes_group1]
    push edi
    or ecx,-1
    xor eax,eax
    repnz scasb
    not ecx
    dec ecx     ;ecx contiene la lunghezza del nome del prefisso

    pop esi
    mov edi,[dest]

    rep movsb   ;copia nel buffer di destinazione il nome del prefisso

    mov eax,offset onebytenames
    mov [dest],edi

    push offset onebytesyntax

    jmp loc1

  group2: ;Segment Override Prefix
    mov eax,[edx*8 + ecx + 4] ;eax = Opcode Flags in lookup table (2nd dword)
    mov [segoverride],eax

    mov eax,offset onebytenames
    push offset onebytesyntax

    jmp loc1
  group3: ;Operand-size Override Prefix
    not [OpSizeAttr]
    mov eax,offset onebytenames
    push offset onebytesyntax
    jmp loc1
  group4: ;Address-size Override Prefix
    not [AddrSizeAttr]
    mov eax,offset onebytenames
    push offset onebytesyntax
    jmp loc1
    
    ret

Invalid:    ;opcode non valida o non supportata

    mov dword ptr[edi],"AVNI"
    mov dword ptr[edi+4],"DIL"

    mov eax,2 ;assume 2 come lunghezza dell'opcode
    
    ret

Escape:     ;0F o escape FPU
    inc ebx
    inc [len]

    cmp dl,0Fh
    jz twobyte

FPU_instruction: ;istruzione FPU (il primo byte  compreso tra D8 e DF)
    movzx eax,byte ptr[ebx] ;al = ModR/M byte
    cmp al,0BFh
    ja ModRM_C0toFF

 ModRM_0toBF:
    and al,00111000b
    shr al,1 ;eax = nnn field * 4
    sub dl,0D8h
    shl dl,5
    add al,dl
    add eax,offset DE_FPU_names_0toBF
    mov edi,[eax]
    test edi,edi
    jz Invalid

    push edi
    
    or ecx,-1
    xor eax,eax
    repne scasb
    not ecx
    dec ecx

    pop esi
    mov edi,[dest]
    rep movsb


    push edi
    push [segoverride]  ;segcode
    push [AddrSizeAttr] ;addrsize
    push [OpSizeAttr]   ;opsize
    push NULL   ;OpFlags
    push NULL   ;Flags
    push ebx    ;Address
    call ManageModRM

    add eax,[len]
    inc eax
    mov byte ptr[edi],0
    ret

 ModRM_C0toFF:
    sub dl,0D8h
    sub al,0C0h
    shl edx,8
    shl eax,2
    mov edi,[eax + edx + offset DE_FPU_names_C0toFF]
    test edi,edi
    jz Invalid

    push edi
    
    or ecx,-1
    xor eax,eax
    repne scasb
    not ecx

    pop esi
    mov edi,[dest]
    rep movsb

    mov eax,[len]
    inc eax

    ret

twobyte:    ;istruzione di 2 bytes che inizia con 0F
    mov eax,offset twobytenames
    push offset twobytesyntax
    jmp loc1
    

Disasm endp

;Gestisce il reg field.
;Richiede 5 parametri:
; - Address: indirizzo del ModR/M byte
; - Flags: flags dell'istruzione nella lookup table
; - OpFlags: flags relativi all'operando
; - opsize: DE_OPSIZESET se l'operand size attribute  settato, DE_OPSIZERESET in caso contrario
; - dest: offset della stringa di destinazione
;Modifica edi ed EFLAGS
;Assume che il Direction Flag sia azzerato
;
;Restituisce in edi l'offset del byte che segue l'ultimo byte scritto

ManageRegField proc Address:DWORD, Flags:DWORD, OpFlags:DWORD, opsize:DWORD, dest:DWORD
    push eax
    push esi

    mov eax,dword ptr[Flags]

    test eax,DE_G
    jnz generalregister

    test eax,DE_C
    jnz controlregister

    test eax,DE_D
    jnz debugregister

    test eax,DE_P
    jnz mmxregister

    test eax,DE_S
    jnz segmentregister

    test eax,DE_T
    jnz testregister

    ;test eax,DE_V
    ;jnz xmmregister

;xmmregister:  ;non utilizzato; incluso solo per possibili usi futuri
;    mov edi,offset DE_xmm_regs
;    jmp @F

generalregister:

    mov eax,dword ptr[OpFlags]
    and al,111b
    cmp al,DE_OP_b
    je gen_reg_8bit
;    cmp al,DE_OP_c
;    je byte_or_word
    cmp al,DE_OP_d
    je gen_reg_32bit
    cmp al,DE_OP_v
    je word_or_dword
    cmp al,DE_OP_w
    je gen_reg_16bit
    
;  byte_or_word:
;    cmp [opsize],DE_OPSIZERESET
;    jz gen_reg_8bit
;    jmp gen_reg_16bit

  word_or_dword:
    cmp [opsize],DE_OPSIZERESET
    jz gen_reg_16bit
    jmp gen_reg_32bit
    
  gen_reg_8bit:
    mov esi,offset DE_gen_regs_8
    jmp @F
  gen_reg_16bit:
    mov esi,offset DE_gen_regs_16
    jmp @F
  gen_reg_32bit:
    mov esi,offset DE_gen_regs_32
    jmp @F

controlregister:
    mov esi,offset DE_ctrl_regs
    jmp @F
debugregister:
    mov esi,offset DE_dbg_regs
    jmp @F
mmxregister:
    mov esi,offset DE_mmx_regs
    jmp @F
segmentregister:
    mov esi,offset DE_seg_regs
    jmp @F
testregister:
    mov esi,offset DE_test_regs
    jmp @F

@@: mov eax,[Address]
    movzx eax,byte ptr[eax]      ;al = ModR/M byte
    and al,00111000b             ;al = (reg field) * 8

    add esi,eax
    mov edi,[dest]
@@: mov al,byte ptr[esi]
    test al,al
    jz @F
    movsb
    jmp @B
@@:

    pop esi
    pop eax
    ret
  
ManageRegField endp


;Gestisce Mod, R/M, e SIB.
;Richiede sette parametri:
; - Address: offset del ModR/M byte
; - Flags: flags dell'istruzione nella lookup table
; - OpFlags: flags dell'operando
; - opsize: DE_OPSIZESET se l'operand-size attribute  settato, DE_OPSIZERESET se non lo 
; - addrsize: DE_ADDRSIZESET se l'operand-size attribute  settato, DE_ADDRSIZERESET se non lo 
; - segcode: un segment code se un segment-override prefix  present; zero in caso contrario
; - dest: offset della stringa di destinazione
;Modifica eax, edi ed EFLAGS
;Assume che il Direction Flag sia azzerato
;
;Restituisce in edi il byte che segue l'ultimo byte scritto, ed in eax la lunghezza di
;SIB byte pi eventuali displacements (0 se non vi sono n SIB n displacements)

ManageModRM proc Address:DWORD, Flags:DWORD, OpFlags:DWORD, opsize:DWORD, addrsize:DWORD, segcode:DWORD, dest:DWORD
    push ebx
    push ecx
    push edx
    push esi

    xor ebx,ebx

    mov ecx,[Address]

    movzx eax,byte ptr[ecx]      ;al = ModR/M byte

    cmp al,0C0h                  ;se il Mod field  uguale a 11b..
    jae register_operand         ;...l'operando  un registro

memory_operand:                  ;operando di memoria
    mov edi,[dest]

    ;--> Gestione Segment-override prefix
    mov esi,[segcode]
    test esi,esi
    jz @F
    lea esi,[esi*4 + offset DE_segoverride_table]
    movsd
    dec edi      ;decrementa edi perch i bytes da copiare sono 3, e non 4
    @@:
    ;<--
    
    mov byte ptr [edi],'['
    inc edi
    mov esi,dword ptr[addrsize]
    test esi,esi
    jnz mem_32
  mem_16:
    and al,11000111b     ;azzera il reg field
    cmp al,00000110b     ;se il Mod  00b e il R/M field  110b...
    jz mem16_disp16      ;...salta a mem16_disp16
    shl al,3             ;tiene solo il R/M field e lo moltiplica per 8
    lea esi,[offset DE_addressing_16bit + eax]

    @@:
    mov al,byte ptr[esi]
    test al,al
    jz @F
    movsb
    jmp @B
    @@:

    xor ebx,ebx

    mov al,byte ptr[ecx] ;al = ModR/M byte
    and al,11000000b     ;tiene solo il Mod field
    jz mem_end           ;se  zero abbiamo finito

    mov al,"+"           ;altrimenti manca ancora il displacement
    stosb

    cmp al,10000000b     ;se  10b...
    jz mem16_disp16      ;...abbiamo un displacement di 16 bit...
    
   mem16_disp8:          ;...altrimenti (cio se  01b) abbiamo un displacement di 8 bit
    movzx eax,byte ptr[ecx+1]
    call ConvertByteToHex
    inc ebx
    jmp mem_end
   mem16_disp16:
    movzx eax,word ptr[ecx+1]    ;ax = displacement
    call ConvertWordToHex
    add ebx,2
    jmp mem_end

  mem_32:
    cmp al,01000000b
    jb Mod_00
    cmp al,10000000b
    jb Mod_01
    jmp Mod_10
   Mod_00:
        mov al,byte ptr[ecx] ;al = ModR/M byte
        and al,111b          ;al = R/M field
        cmp al,4
        jz Mod_00_RM_4
        cmp al,5
        jz Mod_00_RM_5
        Mod_00_RM_General:
            lea esi,[eax*8 + offset DE_gen_regs_32]
         @@:mov al,byte ptr[esi]
            test al,al
            jz @F
            movsb
            jmp @B
         @@:xor ebx,ebx
            jmp mem_end
        Mod_00_RM_4:    ;istruzione con SIB byte
            mov eax,dword ptr[Address]
            call ManageSIB
            lea ebx,[eax+1]
            jmp mem_end
        Mod_00_RM_5:    ;displacement only
            mov eax,dword ptr[ecx+1]
            call ConvertDwordToHex
            mov ebx,4
            jmp mem_end
   Mod_01:
        mov al,byte ptr[ecx] ;al = ModR/M byte
        and al,111b          ;al = R/M field
        cmp al,4
        jz Mod_01_RM_4
        Mod_01_RM_General:
            lea esi,[eax*8 + offset DE_gen_regs_32]
         @@:mov al,byte ptr[esi]
            test al,al
            jz @F
            movsb
            jmp @B
         @@:xor ebx,ebx
            xor eax,eax
            jmp @F
        Mod_01_RM_4:    ;istruzione con SIB byte
            mov eax,dword ptr[Address]
            call ManageSIB
            lea ebx,[eax+1]

        @@: inc ebx
            mov byte ptr[edi],'+'
            inc edi

            mov al,byte ptr[ecx+eax+2]
            call ConvertByteToHex
            jmp mem_end
        
   Mod_10:
        mov al,byte ptr[ecx] ;al = ModR/M byte
        and al,111b          ;al = R/M field
        cmp al,4
        jz Mod_10_RM_4
        Mod_10_RM_General:
            lea esi,[eax*8 + offset DE_gen_regs_32]
         @@:mov al,byte ptr[esi]
            test al,al
            jz @F
            movsb
            jmp @B
         @@:xor ebx,ebx
            xor eax,eax
            jmp @F
        Mod_10_RM_4:    ;istruzione con SIB byte
            mov eax,dword ptr[Address]
            call ManageSIB
            lea ebx,[eax+1]

        @@: add ebx,4
            mov byte ptr[edi],'+'
            inc edi

            mov eax,dword ptr[ecx+eax+2]
            call ConvertDwordToHex
            jmp mem_end   
  mem_end:
    mov al,']'
    stosb

    mov eax,ebx
    jmp return

register_operand:
    movzx eax,byte ptr[ecx] ;al = ModR/M byte
    and al,0111b
    shl al,3     ;eax = R/M * 8
    mov edx,dword ptr[Flags]
    mov ecx,dword ptr[OpFlags]

    test edx,DE_E
    jnz generalregister

    test edx,DE_Q
    jnz mmxregister

    ;test edx,DE_W
    ;jnz xmmregister

;xmmregister:
;    mov edi,offset DE_xmm_regs
;    jmp @F

generalregister:
    and cl,7
    cmp cl,DE_OP_b
    je gen_reg_8bit
;    cmp cl,DE_OP_c
;    je byte_or_word
    cmp cl,DE_OP_d
    je gen_reg_32bit
    cmp cl,DE_OP_v
    je word_or_dword
    cmp cl,DE_OP_w
    je gen_reg_16bit
    
;  byte_or_word:
;    mov ebx,[opsize]
;    cmp ebx,DE_OPSIZERESET
;    jz gen_reg_8bit
;    jmp gen_reg_16bit

  word_or_dword:
    mov ebx,[opsize]
    cmp ebx,DE_OPSIZERESET
    jz gen_reg_16bit
    jmp gen_reg_32bit
    
  gen_reg_8bit:
    mov esi,offset DE_gen_regs_8
    jmp @F
  gen_reg_16bit:
    mov esi,offset DE_gen_regs_16
    jmp @F
  gen_reg_32bit:
    mov esi,offset DE_gen_regs_32
    jmp @F

mmxregister:
    mov esi,offset DE_mmx_regs

@@: add esi,eax
    mov edi,[dest]
@@: mov al,byte ptr[esi]
    test al,al
    jz @F
    movsb
    jmp @B
@@: xor eax,eax


return:
    pop esi
    pop edx
    pop ecx
    pop ebx
    ret

ManageModRM endp

;Gestisce il SIB byte.
;Input:
;  eax = offset del ModR/M byte
;  edi = offset dove il risultato deve essere memorizzato
;Output:
;  eax = larghezza del displacement (se presente)
;  edi = offset del byte che segue l'ultimo byte scritto
ManageSIB proc
    push ecx
    push esi
    movzx ecx, byte ptr[eax+1] ;ecx = SIB byte
    push ecx
    and cl,11111000b           ;ecx = (Scale_Index) * 8
    lea esi,[offset DE_SIB_table + ecx]
  @@:
    mov cl,byte ptr[esi]
    jecxz @F
    mov byte ptr[edi],cl
    inc esi
    inc edi
    jmp @B
  @@:
    mov cl,byte ptr[eax]
    cmp cl,01000000b ;Mod == 0 ?
    pop ecx
    jnb @F
    and cl,111b      ;ecx = Base field
    cmp cl,5
    jne @F
    ;se il mod  00b e il Base field  101b, abbiamo un disp32 subito dopo il SIB...
    mov eax, dword ptr[eax+2] ;eax = displacement
    call ConvertDwordToHex
    mov eax, 4
    jmp SIB_end

  @@:
    ;..altrimenti decodifichiamo normalmente la base come un general-register a 32 bit
    and cl,111b
    lea esi,[offset DE_gen_regs_32 + ecx*8]
    movsd      ;copia
    dec edi    ;3 bytes
    xor eax,eax
 SIB_end:
    pop esi
    pop ecx
    ret
ManageSIB endp

;Converte AL in esadecimale e salva il risultato nella memoria puntata da edi
;Input: 
;  AL = byte da convertire
; EDI = offset dove il risultato deve essere salvato
;Output:
; EDI = offset del byte che segue l'ultimo byte scritto
ConvertByteToHex proc
    push eax
    movzx eax,al

    push eax

    shr eax,4

    add al,90h
    daa
    adc al,40h
    daa

    stosb

    pop eax

    and al,0Fh

    add al,90h
    daa
    adc al,40h
    daa
    
    stosb

    pop eax

    ret
ConvertByteToHex endp

;Converte AX in esadecimale e salva il risultato nella memoria puntata da edi
;Input: 
;  AX = word da convertire
; EDI = offset dove il risultato deve essere salvato
;Output:
; EDI = offset del byte che segue l'ultimo byte scritto
ConvertWordToHex proc
    ror eax,8
    call ConvertByteToHex
    rol eax,8
    call ConvertByteToHex
    ret
ConvertWordToHex endp

;Converte EAX in esadecimale e salva il risultato nella memoria puntata da edi
;Input: 
; EAX = dword da convertire
; EDI = offset dove il risultato deve essere salvato
;Output:
; EDI = offset del byte che segue l'ultimo byte scritto
ConvertDwordToHex proc
    ror eax,16
    call ConvertWordToHex
    rol eax,16
    call ConvertWordToHex
    ret
ConvertDwordToHex endp


end DllEntry